@@ -1,5 +1,5 @@
 /*
- * This file is part of liblcf. Copyright (c) 2017 liblcf authors.
+ * This file is part of liblcf. Copyright (c) 2018 liblcf authors.
  * https://github.com/EasyRPG/liblcf - https://easyrpg.org
  *
  * liblcf is Free/Libre Open Source Software, released under the MIT License.
@@ -62,20 +62,20 @@ template <> struct TypeCategory<RPG::SavePicture::Flags>			{ static const Catego
 template <> struct TypeCategory<RPG::Equipment>					{ static const Category::Index value = Category::RawStruct; };
 template <> struct TypeCategory<RPG::EventCommand>				{ static const Category::Index value = Category::RawStruct; };
 template <> struct TypeCategory<RPG::MoveCommand>				{ static const Category::Index value = Category::RawStruct; };
-template <> struct TypeCategory<RPG::Parameters>				{ static const Category::Index value = Category::RawStruct; };
+template <> struct TypeCategory<RPG::Parameters>					{ static const Category::Index value = Category::RawStruct; };
 template <> struct TypeCategory<RPG::TreeMap>					{ static const Category::Index value = Category::RawStruct; };
 template <> struct TypeCategory<RPG::Rect>						{ static const Category::Index value = Category::RawStruct; };
 
 template <>	struct TypeCategory<uint8_t>						{ static const Category::Index value = Category::Primitive; };
 template <>	struct TypeCategory<int16_t>						{ static const Category::Index value = Category::Primitive; };
 template <>	struct TypeCategory<uint32_t>						{ static const Category::Index value = Category::Primitive; };
-template <>	struct TypeCategory<int>							{ static const Category::Index value = Category::Primitive; };
+template <>	struct TypeCategory<int32_t>						{ static const Category::Index value = Category::Primitive; };
 template <>	struct TypeCategory<bool>							{ static const Category::Index value = Category::Primitive; };
 template <>	struct TypeCategory<double>							{ static const Category::Index value = Category::Primitive; };
 template <>	struct TypeCategory<std::string>					{ static const Category::Index value = Category::Primitive; };
 
 template <class T>
-struct TypeCategory<std::vector<T> > {
+struct TypeCategory<std::vector<T>> {
 	static const Category::Index value = TypeCategory<T>::value;
 };
 
@@ -174,14 +174,14 @@ struct Primitive {
  * Vector specialization.
  */
 template <class T>
-struct Primitive<std::vector<T> > {
+struct Primitive<std::vector<T>> {
 	static void ReadLcf(std::vector<T>& ref, LcfReader& stream, uint32_t length) {
 		stream.Read(ref, length);
 #ifdef LCF_DEBUG_TRACE
 		typename std::vector<T>::iterator it;
 		printf("  ");
 		for (it = ref.begin(); it != ref.end(); ++it) {
-			printf("%d, ", *it);
+			printf("%d, ", static_cast<int>(*it));
 		}
 		printf("\n");
 #endif
@@ -204,8 +204,8 @@ struct Primitive<std::vector<T> > {
  * Int specialization.
  */
 template <>
-struct Primitive<int> {
-	static void ReadLcf(int& ref, LcfReader& stream, uint32_t length) {
+struct Primitive<int32_t> {
+	static void ReadLcf(int32_t& ref, LcfReader& stream, uint32_t length) {
 		if (length >= 1 && length <= 5) {
 			ref = stream.ReadInt();
 #ifdef LCF_DEBUG_TRACE
@@ -220,16 +220,16 @@ struct Primitive<int> {
 		}
 
 	}
-	static void WriteLcf(const int& ref, LcfWriter& stream) {
+	static void WriteLcf(const int32_t& ref, LcfWriter& stream) {
 		stream.WriteInt(ref);
 	}
-	static int LcfSize(const int& ref, LcfWriter& /* stream */) {
+	static int LcfSize(const int32_t& ref, LcfWriter& /* stream */) {
 		return LcfReader::IntSize(ref);
 	}
-	static void WriteXml(const int& ref, XmlWriter& stream) {
+	static void WriteXml(const int32_t& ref, XmlWriter& stream) {
 		stream.WriteInt(ref);
 	}
-	static void ParseXml(int& ref, const std::string& data) {
+	static void ParseXml(int32_t& ref, const std::string& data) {
 		XmlReader::Read(ref, data);
 	}
 };
@@ -291,8 +291,9 @@ template <class S>
 struct Field {
 	typedef S struct_type;
 
-	int id;
 	const char* const name;
+	int id;
+	bool is2k3;
 
 	virtual void ReadLcf(S& obj, LcfReader& stream, uint32_t length) const = 0;
 	virtual void WriteLcf(const S& obj, LcfWriter& stream) const = 0;
@@ -302,8 +303,8 @@ struct Field {
 	virtual void BeginXml(S& obj, XmlReader& stream) const = 0;
 	virtual void ParseXml(S& obj, const std::string& data) const = 0;
 
-	Field(int id, const char* name) :
-		id(id), name(name) {}
+	Field(int id, const char* name, bool is2k3) :
+		name(name), id(id), is2k3(is2k3) {}
 };
 
 // Equivalence traits
@@ -327,7 +328,7 @@ struct Compare_Test {
 };
 
 template <class T>
-struct Compare_Test<std::vector<T> > {
+struct Compare_Test<std::vector<T>> {
 	static const bool value = Compare_Test<T>::value;
 };
 
@@ -399,8 +400,8 @@ struct TypedField : public Field<S> {
 		return Compare_Traits<T>::IsEqual(a.*ref, b.*ref);
 	}
 
-	TypedField(T S::*ref, int id, const char* name) :
-		Field<S>(id, name), ref(ref) {}
+	TypedField(T S::*ref, int id, const char* name, bool is2k3) :
+		Field<S>(id, name, is2k3), ref(ref) {}
 };
 
 /**
@@ -411,15 +412,15 @@ struct SizeField : public Field<S> {
 	const std::vector<T> S::*ref;
 
 	void ReadLcf(S& /* obj */, LcfReader& stream, uint32_t length) const {
-		int dummy;
-		TypeReader<int>::ReadLcf(dummy, stream, length);
+		int32_t dummy;
+		TypeReader<int32_t>::ReadLcf(dummy, stream, length);
 	}
 	void WriteLcf(const S& obj, LcfWriter& stream) const {
-		int size = TypeReader<std::vector<T> >::LcfSize(obj.*ref, stream);
-		TypeReader<int>::WriteLcf(size, stream);
+		int size = TypeReader<std::vector<T>>::LcfSize(obj.*ref, stream);
+		TypeReader<int32_t>::WriteLcf(size, stream);
 	}
 	int LcfSize(const S& obj, LcfWriter& stream) const {
-		int size = TypeReader<std::vector<T> >::LcfSize(obj.*ref, stream);
+		int size = TypeReader<std::vector<T>>::LcfSize(obj.*ref, stream);
 		return LcfReader::IntSize(size);
 	}
 	void WriteXml(const S& /* obj */, XmlWriter& /* stream */) const {
@@ -435,8 +436,8 @@ struct SizeField : public Field<S> {
 		return (a.*ref).empty() && (b.*ref).empty();
 	}
 
-	SizeField(const std::vector<T> S::*ref, int id) :
-		Field<S>(id, ""), ref(ref) {}
+	SizeField(const std::vector<T> S::*ref, int id, bool is2k3) :
+		Field<S>(id, "", is2k3), ref(ref) {}
 };
 
 /**
@@ -588,26 +589,21 @@ struct TypeReader<std::vector<T>, Category::Struct> {
 	}
 };
 
+
+
 /**
  * Flags class template.
  */
 template <class S>
 class Flags {
-public:
-	struct Flag {
-		Flag(bool S::*ref, const char* const name) : ref(ref), name(name) {}
-		bool S::*ref;
-		const char* const name;
-	};
-
 private:
-	static const uint32_t max_size;
-	typedef std::map<const char* const, const Flag*, StringComparator> tag_map_type;
-	static const Flag* flags[];
-	static tag_map_type tag_map;
 	static const char* const name;
+	static constexpr size_t num_flags = std::tuple_size<decltype(S::flags)>::value;
+	static const std::array<const char* const, num_flags> flag_names;
+	static const std::array<bool, num_flags> flags_is2k3;
 
-	static void MakeTagMap();
+	static const char* tag(int idx);
+	static int idx(const char* tag);
 
 	template <class T> friend class FlagsXmlHandler;
 
@@ -620,7 +616,19 @@ public:
 };
 
 template <class S>
-std::map<const char* const, const typename Flags<S>::Flag*, StringComparator> Flags<S>::tag_map;
+inline const char* Flags<S>::tag(int idx) {
+	return Flags<S>::flag_names[idx];
+}
+
+template <class S>
+inline int Flags<S>::idx(const char* tag) {
+	for (size_t i = 0; i < flag_names.size(); ++i) {
+		if (std::strcmp(flag_names[i], tag) == 0) {
+			return i;
+		}
+	}
+	return -1;
+}
 
 /**
  * Wrapper XML handler struct.
@@ -702,17 +710,19 @@ private:
 #define LCF_STRUCT_FIELDS_END() \
 	NULL }; \
 
-#define LCF_STRUCT_TYPED_FIELD(T, REF) \
+#define LCF_STRUCT_TYPED_FIELD(T, REF, IS2K3) \
 	new TypedField<RPG::LCF_CURRENT_STRUCT, T>( \
 		  &RPG::LCF_CURRENT_STRUCT::REF \
 		, LCF_CHUNK_SUFFIX::BOOST_PP_CAT(Chunk, LCF_CURRENT_STRUCT)::REF \
 		, BOOST_PP_STRINGIZE(REF) \
+		, IS2K3 \
 	) \
 
-#define LCF_STRUCT_SIZE_FIELD(T, REF) \
+#define LCF_STRUCT_SIZE_FIELD(T, REF, IS2K3) \
 	new SizeField<RPG::LCF_CURRENT_STRUCT, T>( \
 		  &RPG::LCF_CURRENT_STRUCT::REF \
 		, LCF_CHUNK_SUFFIX::BOOST_PP_CAT(Chunk, LCF_CURRENT_STRUCT)::BOOST_PP_CAT(REF, _size) \
+		, IS2K3 \
 	) \
 
 #endif
