@@ -1,5 +1,5 @@
 /*
- * This file is part of liblcf. Copyright (c) 2017 liblcf authors.
+ * This file is part of liblcf. Copyright (c) 2018 liblcf authors.
  * https://github.com/EasyRPG/liblcf - https://easyrpg.org
  *
  * liblcf is Free/Libre Open Source Software, released under the MIT License.
@@ -16,7 +16,7 @@
 #if defined(LCF_SUPPORT_XML)
 #  include <expat.h>
 #endif
-#include "reader_types.h"
+#include <stdint.h>
 #include "lcf_options.h"
 #include "reader_util.h"
 
