@@ -1,5 +1,5 @@
 /*
- * This file is part of liblcf. Copyright (c) 2017 liblcf authors.
+ * This file is part of liblcf. Copyright (c) 2018 liblcf authors.
  * https://github.com/EasyRPG/liblcf - https://easyrpg.org
  *
  * liblcf is Free/Libre Open Source Software, released under the MIT License.
@@ -14,7 +14,7 @@
 #include <vector>
 #include <memory>
 #include <cstdio>
-#include "reader_types.h"
+#include <stdint.h>
 
 /**
  * XmlWriter class.
